param
(
    [switch]$Build,
    [switch]$BuildAndPublish,
    [switch]$PublishAndDeploy,
    [switch]$CopyAndDeploy,

    [Parameter(Mandatory=$true)]
    [string] $Environment,

    [string]$WebSiteHost,
    [string]$WebsitesRootDir,
    [string]$DeployTag,
    [string]$PublishOutput
)
$ErrorActionPreference = "stop"

if ($Build) {
    # Display variables
    Write-Host ("Environment = {0}" -f $Environment)

    # Start build process.
    & dotnet restore src\KitemanaLivestream.csproj
    if(!$?) { Exit $LASTEXITCODE }

    & dotnet build --configuration Release --no-restore src\KitemanaLivestream.csproj -p:EnvironmentName=$Environment
    if(!$?) { Exit $LASTEXITCODE }
}

if ($BuildAndPublish) {
    # Display variables
    Write-Host ("Environment = {0}" -f $Environment)
    Write-Host ("PublishOutput = {0}" -f $PublishOutput)

    # Start build process.
    & dotnet restore src\KitemanaLivestream.csproj
    if(!$?) { Exit $LASTEXITCODE }

    # Build the solution
    & dotnet build --configuration Release --no-restore src\KitemanaLivestream.csproj -p:EnvironmentName=$Environment
    if(!$?) { Exit $LASTEXITCODE }

    # Publish the solution
    & dotnet publish --configuration Release --no-build --no-self-contained -p:EnvironmentName=$Environment --output $PublishOutput src\KitemanaLivestream.csproj
    if(!$?) { Exit $LASTEXITCODE }
}

if ($PublishAndDeploy) {
    # Display variables
    Write-Host ("Environment = {0}" -f $Environment)
    Write-Host ("WebSitesRootDir = {0}" -f $WebSitesRootDir)
    Write-Host ("WebSiteHost = {0}" -f $WebSiteHost)
    Write-Host ("DeployTag = {0}" -f $DeployTag)

    #Set pre-build variables
    $WebSiteFullName = "gitlab-deploy-{0}-{1}" -f $CI_PROJECT_ROOT_NAMESPACE, $CI_PROJECT_NAME
    $DeployTargetDir = $(Join-Path "$WebSitesRootDir" "$WebSiteFullName")
    $DeployTargetDir = $(Join-Path "$DeployTargetDir" "$DeployTag")

    # Display variables
    Write-Host ("WebSiteFullName = {0}" -f $WebSiteFullName)
    Write-Host ("DeployTargetDir = {0}" -f $DeployTargetDir)

    # Starting publish
    & dotnet publish --configuration Release --no-build --no-self-contained -p:EnvironmentName=$Environment --output "$DeployTargetDir" src\KitemanaLivestream.csproj
    if(!$?) { Exit $LASTEXITCODE }

    # Swtich website physical path to new version
    Import-Module WebAdministration

    $website = Get-Website -Name $WebSiteFullName
    if ($website -eq $null -or $website -eq '') {
        New-Website -Name $WebSiteFullName -HostHeader $WebSiteHost -PhysicalPath $DeployTargetDir
    } else {
        Set-ItemProperty ('IIS:\Sites\' + $WebSiteFullName) -Name physicalPath -Value $DeployTargetDir
    }    
}

if ($CopyAndDeploy) {
    # Display variables
    Write-Host ("Environment = {0}" -f $Environment)
    Write-Host ("WebSitesRootDir = {0}" -f $WebSitesRootDir)
    Write-Host ("WebSiteHost = {0}" -f $WebSiteHost)
    Write-Host ("DeployTag = {0}" -f $DeployTag)
    Write-Host ("PublishOutput = {0}" -f $PublishOutput)

    #Set pre-build variables
    $WebSiteFullName = "gitlab-deploy-{0}-{1}" -f $CI_PROJECT_ROOT_NAMESPACE, $CI_PROJECT_NAME
    $DeployTargetDir = $(Join-Path "$WebSitesRootDir" "$WebSiteFullName")
    $DeployTargetDir = $(Join-Path "$DeployTargetDir" "$DeployTag")

    # Display variables
    Write-Host ("WebSiteFullName = {0}" -f $WebSiteFullName)
    Write-Host ("DeployTargetDir = {0}" -f $DeployTargetDir)

    # Create destination folder.
    $folderExists = Test-Path -Path $DeployTargetDir
    if ($folderExists -eq $false) {
        New-Item -Path $DeployTargetDir -ItemType Directory
    }

    # Copy published outputs to website target physical path
    robocopy $PublishOutput $DeployTargetDir /E /COPYALL

    # Swtich website physical path to new version
    Import-Module WebAdministration

    $website = Get-Website -Name $WebSiteFullName
    if ($website -eq $null -or $website -eq '') {
        New-Website -Name $WebSiteFullName -HostHeader $WebSiteHost -PhysicalPath $DeployTargetDir
    } else {
        Set-ItemProperty ('IIS:\Sites\' + $WebSiteFullName) -Name physicalPath -Value $DeployTargetDir
    }    
}
